#include "method.hpp"
#include "../string.hpp"

namespace fz::authentication {

const available_methods available_methods::none;

available_methods available_methods::filter(const methods_set &set) const
{
	available_methods ret;

	for (const auto &s: *this) {
		if ((s & set) == set)
			ret.push_back(s);
	}

	return ret;
}

bool available_methods::has(const methods_set &set) const
{
	for (const auto &s: *this) {
		if (s == set)
			return true;
	}

	return false;
}


bool available_methods::can_verify(const methods_set &methods) const
{
	for (const auto &s: *this) {
		// Anything can verify none.
		// All the rest can verify only itself
		if (!s || (methods && (s & methods) == methods))
			return true;
	}

	return false;

}

bool available_methods::set_verified(const any_method &method)
{
	for (auto &s: *this) {
		s.erase(method);

		if (!s) {
			assign({ {} });
			return false;
		}
	}

	return !empty();
}

bool available_methods::is_auth_necessary() const
{
	for (const auto &s: *this) {
		if (!s)
			return false;
	}

	return !empty();
}

bool available_methods::is_auth_possible() const
{
	return !empty();
}

std::string to_string(const available_methods &am)
{
	return fz::join<std::string>(am, ",");
}

std::string to_string(const any_method &m)
{
	return fz::sprintf("{ id = %d, set = %s }", m.index(), methods_set(m));
}

std::string to_string(const methods_list &ml)
{
	return fz::sprintf("(%s)", fz::join<std::string>(ml, ","));
}

std::wstring to_wstring(const available_methods &am)
{
	return fz::join<std::wstring>(am, L",");
}

std::wstring to_wstring(const any_method &m)
{
	return fz::sprintf(L"{ id = %d, set = %s }", m.index(), methods_set(m));
}

std::wstring to_wstring(const methods_list &ml)
{
	return fz::sprintf(L"(%s)", fz::join<std::wstring>(ml, L","));
}

}
