#ifndef FZ_HTTP_RESPONSE_HPP
#define FZ_HTTP_RESPONSE_HPP

#include <string>
#include <functional>

#include <libfilezilla/event_handler.hpp>

#include "headers.hpp"

namespace fz::http
{

struct response
{
	unsigned int code;
	std::string reason;
	std::string body;
	http::headers headers;

public:
	using event = simple_event<response, response>;
	using handler = std::function<void(response &&)>;

	static handler forward_as_event(event_handler &eh)
	{
		return [&eh](auto && response) {
			eh.send_event<event>(std::move(response));
		};
	}

	enum code_type_t
	{
		informational = 1,
		successful,
		redirect,
		client_error,
		server_error,
		invalid = 999
	};

	code_type_t code_type() const
	{
		return 100 > code || code > 599 ? invalid : code_type_t(code/100);
	}
};

}

#endif // FZ_HTTP_RESPONSE_HPP
