#ifndef NOTIFIER_HPP
#define NOTIFIER_HPP

#include <string_view>

#include <libfilezilla/file.hpp>
#include <libfilezilla/logger.hpp>

namespace fz {

class notifier {
public:

	enum class entry_transfer_status: std::uint8_t { started, stopped };
	enum class progress_type: std::uint8_t { sent_data, received_data, sent_control, received_control };

	struct protocol_info
	{
		enum status
		{
			insecure,
			not_completely_secure,
			secure
		};

		virtual ~protocol_info(){}

		virtual status get_status() const = 0;
		virtual std::string get_name() const = 0;

	public:
		protocol_info() = default;
		protocol_info(const protocol_info &) = default;
		protocol_info(protocol_info &&) = default;
		protocol_info &operator=(const protocol_info &) = default;
		protocol_info &operator=(protocol_info &&) = default;
	};

	virtual ~notifier(){}

	virtual void notify_user_name(std::string_view name) = 0;
	virtual void notify_entry_info(std::string_view path, file::mode mode, std::int64_t size, std::int64_t resume_offset) = 0;
	virtual void notify_entry_transfer_status(entry_transfer_status status, int error) = 0;
	virtual void notify_progress(progress_type type, std::int64_t amount) = 0;
	virtual void notify_protocol_info(const protocol_info &info) = 0;

	virtual logger_interface &logger() = 0;
};

}

#endif // NOTIFIER_HPP
