#include <libfilezilla/util.hpp>

#include "../../filezilla/tcp/session.hpp"

namespace fz::tcp {

session::session(event_handler &target_handler, id id)
	: target_handler_(target_handler)
	, id_(id)
{
}

session::~session()
{
}

session::id session::get_id() const
{
	return id_;
}

session::factory::~factory()
{
}

void session::factory::listener_status_changed(const listener &)
{
}

bool session::factory::log_on_session_exit()
{
	return true;
}

session::factory::base::base(event_loop_pool &pool, tcp::address_list &disallowed_ips, tcp::address_list &allowed_ips, authentication::autobanner &autobanner, logger_interface &logger)
	: pool_(pool)
	, disallowed_ips_(disallowed_ips)
	, allowed_ips_(allowed_ips)
	, autobanner_(autobanner)
	, logger_(logger)
{
}

std::unique_ptr<session> session::factory::base::make_session(event_handler &target_handler, session::id id, std::unique_ptr<socket> socket, const address_info &address_info, int &error)
{
	if (!socket || error)
		return {};

	if (autobanner_.is_banned(socket->peer_ip(), socket->address_family())) {
		logger_.log_u(logmsg::debug_warning, L"Address %s has been temporarily banned due to brute force protection. Refusing connection.", socket->peer_ip());
		error = EACCES;
		return {};
	}

	if (disallowed_ips_.contains(socket->peer_ip(), socket->address_family())) {
		if (!allowed_ips_.contains(socket->peer_ip(), socket->address_family())) {
			logger_.log_u(logmsg::debug_warning, L"Address %s has been banned. Refusing connection.", socket->peer_ip());
			error = EACCES;
			return {};
		}
	}

	return make_session(target_handler, pool_.get_loop(), id, std::move(socket), address_info, error);
}

}
