#ifndef FZ_TCP_SESSION_HPP
#define FZ_TCP_SESSION_HPP

#include <libfilezilla/event_handler.hpp>
#include <libfilezilla/logger.hpp>

#include "../channel.hpp"
#include "address_info.hpp"
#include "listener.hpp"
#include "address_list.hpp"
#include "../authentication/autobanner.hpp"
#include "../event_loop_pool.hpp"

namespace fz::tcp {

class session
{
public:
	using id = std::uint64_t;

	class factory;

	using ended_event = simple_event<session, id, channel::error_type /*error*/>;

public:
	session(event_handler &target_handler, id id);
	virtual ~session();

	session(session &&) = delete;
	session(const session &) = delete;

	id get_id() const;
	virtual bool is_alive() const = 0;

	//! Must shut the session down and send an ended_event when done.
	//! Postcondition: is_alive() == false;
	virtual void shutdown(int err = 0) = 0;

protected:
	event_handler &target_handler_;
	id id_;
};

class session::factory
{
public:
	class base;

	virtual ~factory();
	virtual std::unique_ptr<session> make_session(event_handler &target_handler, session::id id, std::unique_ptr<socket> socket, const address_info &address_info, int &error /* In-Out */) = 0;
	virtual void listener_status_changed(const listener &listener);
	virtual bool log_on_session_exit();
};

class session::factory::base: public session::factory
{
public:
	base(event_loop_pool &pool, tcp::address_list &disallowed_ips, tcp::address_list &allowed_ips, authentication::autobanner &autobanner, logger_interface &logger);

	std::unique_ptr<session> make_session(event_handler &target_handler, session::id id, std::unique_ptr<socket> socket, const address_info &address_info, int &error /* In-Out */) override final;
	virtual std::unique_ptr<session> make_session(event_handler &target_handler, event_loop &loop, session::id id, std::unique_ptr<socket> socket, const address_info &address_info, int &error /* In-Out */) = 0;

private:
	fz::mutex mutex_;

	event_loop_pool &pool_;
	tcp::address_list &disallowed_ips_;
	tcp::address_list &allowed_ips_;
	authentication::autobanner &autobanner_;

	logger_interface &logger_;
};

}

#endif // FZ_TCP_PEER_HPP
