#ifndef ADDRESSINFOLISTEDITOR_H
#define ADDRESSINFOLISTEDITOR_H

#include <vector>
#include <wx/panel.h>
#include <wx/weakref.h>

#include "../filezilla/tcp/address_info.hpp"
#include "../filezilla/util/options.hpp"

class wxButton;

class AddressInfoListEditor: public wxPanel
{
	using validator_t = std::function<wxString (const wxString &address, unsigned int port)>;

public:
	struct ListInfo
	{
		/// If nullptr, then the editor won't take this entry in consideration
		std::vector<fz::tcp::address_info> *list{};

		/// If empty, then the editor won't take this entry in consideration.
		/// The first item is the default one.
		std::initializer_list<std::pair<fz::tcp::address_info::tls_mode_t, const wxString &>> tls2name_map{};
	};

	AddressInfoListEditor() = default;

	bool Create(wxWindow *parent, bool edit_proto = true);

	//! Sets the lists into the editor.
	//! It *doesn't* take ownership of the lists.
	void SetLists(std::initializer_list<ListInfo> lists);
	void SetAddressAndPortValidator(validator_t validator);
	wxString GetMatchingAddress(const wxString &address, unsigned int port, bool any_is_equivalent);

	bool Validate() override;
	bool TransferDataFromWindow() override;

private:
	class Grid;
	class Table;

	Grid *grid_;
	Table *table_;
	wxButton *remove_button_;
	wxButton *add_button_;
};

#endif // ADDRESSINFOLISTEDITOR_H
