#include <libfilezilla/glue/wx.hpp>

#include <wx/dcclient.h>
#include <wx/tokenzr.h>

#include "wrappedtext.hpp"
#include "helpers.hpp"

wxWrappedText::wxWrappedText(wxWindow *parent, const wxString &text)
	: wxPanel(parent)
	, text_(new wxStaticText(this, wxID_ANY, wxEmptyString))
{
	wxPanel::SetLabel(text);

	auto min_width = std::min(wxWindowDC(this).GetTextExtent(text).GetWidth(), wxDlg2Px(this, 200));

	text_->SetLabel(text);
	text_->Wrap(min_width);

	auto first_line = wxStringTokenizer(text_->GetLabel(), wxT("\r\n")).GetNextToken();

	min_width = std::min(wxWindowDC(this).GetTextExtent(first_line).GetWidth(), min_width);

	text_->SetLabel(wxEmptyString);

	SetMinClientSize({min_width, -1});
}

void wxWrappedText::SetLabel(const wxString &label)
{
	wxPanel::SetLabel(label);
	InvalidateBestSize();
	GetParent()->Layout();
}

bool wxWrappedText::SetForegroundColour(const wxColour &colour)
{
	return text_->SetForegroundColour(colour);
}

bool wxWrappedText::SetBackgroundColour(const wxColour &colour)
{
	return text_->SetBackgroundColour(colour);
}

bool wxWrappedText::InformFirstDirection(int dir, int size, int other_size)
{
	if (!dir)
		return false;

	if (dir == wxVERTICAL)
		size = other_size;

	if (size != text_->GetSize().GetWidth()) {
		informed_ = true;

		text_->SetLabel(GetLabel());
		text_->Wrap(size);

		return true;
	}

	return false;
}

wxSize wxWrappedText::DoGetBestClientSize() const
{
	if (!informed_) {
		text_->SetLabel(GetLabel());
		text_->Wrap(GetMinClientSize().GetWidth());
	}

	return text_->GetSize();
}

bool wxWrappedText::AcceptsFocus() const
{
	return false;
}
