#ifndef ADMINISTRATOR_NOTIFIER_HPP
#define ADMINISTRATOR_NOTIFIER_HPP

#include "administrator.hpp"

class administrator::log_forwarder: public fz::logger::modularized {
public:
	log_forwarder(administrator &administrator, fz::ftp::session::id session_id);
	log_forwarder(logger_interface &parent, administrator &administrator, fz::ftp::session::id session_id);

	void do_log(fz::logmsg::type t, const info_list &l, std::wstring &&message) override;

	void detach_from_administrator();

private:
	administrator *administrator_;
	fz::ftp::session::id session_id_;

	mutable fz::mutex mutex_;
};

class administrator::notifier: public fz::notifier {
public:
	~notifier() override;

	void notify_user_name(std::string_view name) override;
	void notify_entry_info(std::string_view path, fz::file::mode mode, int64_t size, int64_t resume_offset) override;
	void notify_entry_transfer_status(entry_transfer_status status, int error) override;
	void notify_progress(progress_type type, int64_t amount) override;
	void notify_protocol_info(const protocol_info &info) override;

	fz::logger_interface &logger() override;

	notifier(administrator &administrator, fz::ftp::session::id id, const fz::datetime &start, const std::string &peer_ip, fz::address_type peer_address_type, fz::logger_interface &logger);

	void send_session_info(administration::engine::session &session) const;
	void detach_from_administrator();

private:
	administrator *administrator_;

	std::uint64_t session_id_;
	fz::datetime start_;
	fz::monotonic_clock monotonic_start_;
	std::string peer_ip_;
	fz::address_type peer_address_type_;

	std::string user_name_;
	fz::duration user_name_set_time_{};

	int64_t amount_at_transfer_start_{};
	int64_t progress_amounts_[4]{};

	std::string entry_path_{};
	fz::file::mode entry_mode_{};
	int64_t entry_size_{};
	int64_t entry_resume_offset_{};
	fz::duration entry_info_set_time_{};
	fz::duration entry_transfer_started_time_{};

	std::optional<administration::session::any_protocol_info> proto_info_;
	fz::duration proto_info_set_time_;

	log_forwarder log_forwarder_;

	mutable fz::mutex mutex_;
};

#endif // ADMINISTRATOR_NOTIFIER_HPP
